@@ -1,6 +1,6 @@
 # this code originally by Dan Putler, used with permission 
 
-# last modified 2019-05-15 by J. Fox
+# last modified 2023-08-07 by J. Fox
 
 
 listKmeansSolutions <- function(envir=.GlobalEnv, ...) {
@@ -74,6 +74,10 @@ kmeansClustering <- function () {
         if (clusterAssign == "1") {
             if (is.element(clusterVariable, Variables())) {
                 if ("no" == tclvalue(checkReplace(clusterVariable))) {
+                    if (getRcmdr("onApplyCalled")){
+                        putRcmdr("onApplyCalled", FALSE)
+                        return()
+                    }
                     kmeansClustering()
                     return()
                 }
@@ -98,6 +102,8 @@ kmeansClustering <- function () {
                          ", iter.max = ", iters, ", num.seeds = ", seeds, 
                          ")", sep = "")
         doItAndPrint(paste(".cluster <- ", command))
+        insertRmdSection(paste0(gettextRmdHeader("K-Means Clustering: "),
+                                paste(x, collapse = ", ")))
         if (clusterSummary == "1") {
             doItAndPrint(paste(".cluster$size # Cluster Sizes"))
             doItAndPrint(paste(".cluster$centers # Cluster Centroids"))
@@ -204,7 +210,7 @@ hierarchicalCluster <- function () {
 						initial.dendro = dendro, initial.tab = tab))
 		closeDialog()
 		varFormula <- paste(x, collapse = "+")
-		vars <- paste(x, collapse = ",", sep = "")
+		vars <- paste(x, collapse = ", ", sep = "")
 		.activeDataSet <- ActiveDataSet()
 		dset <- if (subset == gettextRcmdr("<all valid cases>")) 
 					.activeDataSet
@@ -246,6 +252,8 @@ hierarchicalCluster <- function () {
 							dset, "\"", ", sub=", "\"", "Method=", clusMethod, 
 							"; Distance=", distlab, "\"", ")", sep = ""))
 		}
+        insertRmdSection(paste0(gettextRmdHeader("Hiearchical Cluster Analysis: "),
+                                paste(vars, collapse = ", ")))
 		activateMenus()
 		tkfocus(CommanderWindow())
 	}
@@ -416,6 +424,10 @@ appendHclustGroup <- function () {
 		closeDialog()
 		if (is.element(label, Variables())) {
 			if ("no" == tclvalue(checkReplace(label))) {
+			    if (getRcmdr("onApplyCalled")){
+			        putRcmdr("onApplyCalled", FALSE)
+			        return()
+			    }
 				appendHclustGroup()
 				return()
 			}
