@@ -1,6 +1,6 @@
 # Distributions menu dialogs for selecting samples
 
-# last modified 2019-05-15 by J. Fox
+# last modified 2022-06-27 by J. Fox
 # modified by Miroslav M. Ristic (15 January 2011)
 
 
@@ -66,6 +66,10 @@ distributionSamples <- function(nameVar) {
     }
     if (is.element(dsnameValue, listDataSets())) {
       if ("no" == tclvalue(checkReplace(dsnameValue, gettextRcmdr("Data set")))){
+          if (getRcmdr("onApplyCalled")){
+              putRcmdr("onApplyCalled", FALSE)
+              return()
+          }
         nameVarF()
         return()
       }
@@ -139,6 +143,8 @@ distributionSamples <- function(nameVar) {
     }
     command.3 <- paste(command.3, "\n})", sep="")
     if (any.summaries) doItAndPrint(command.3)
+    title <- if (nameVar != "t") tools::toTitleCase(nameVar) else "t"
+    insertRmdSection(gettextRmdHeader(paste0("Sample from ",  title, " Distribution")))
     activeDataSet(dsnameValue)
     tkfocus(CommanderWindow())
     putDialog(dialogName, list(initialValues=vars, dsname=dsnameValue,
