@@ -1,6 +1,6 @@
 # Statistics Menu dialogs
 
-# last modified 2022-06-15 by J. Fox
+# last modified 2023-08-07 by J. Fox
 
     # Models menu
 
@@ -85,6 +85,10 @@ linearRegressionModel <- function () {
     if (is.element(modelValue, listLinearModels())) {
       if ("no" == tclvalue(checkReplace(modelValue, type = gettextRcmdr("Model")))) {
         UpdateModelNumber(-1)
+          if (getRcmdr("onApplyCalled")){
+              putRcmdr("onApplyCalled", FALSE)
+              return()
+          }
         linearRegressionModel()
         return()
       }
@@ -158,12 +162,13 @@ linearModel <- function(){
     if (trim.blanks(subset) == gettextRcmdr("<all valid cases>") || trim.blanks(subset) == ""){
       subset <- ""
     }
-    else{
-      subset <- paste(", subset=", subset, sep="")
-    }
+    # else{
+    #   subset <- paste(", subset=", subset, sep="")
+    # }
     weight.var <- getSelection(weightComboBox)
     weights <- if (weight.var == gettextRcmdr("<no variable selected>")) ""
-    else paste(", weights=", weight.var, sep="")
+#    else paste(", weights=", weight.var, sep="")
+    else weight.var
     check.empty <- gsub(" ", "", tclvalue(lhsVariable))
     if ("" == check.empty) {
       errorCondition(recall=linearModel, message=gettextRcmdr("Left-hand side of model empty."), model=TRUE)
@@ -177,6 +182,10 @@ linearModel <- function(){
     if (is.element(modelValue, listLinearModels())) {
       if ("no" == tclvalue(checkReplace(modelValue, type=gettextRcmdr("Model")))){
         UpdateModelNumber(-1)
+          if (getRcmdr("onApplyCalled")){
+              putRcmdr("onApplyCalled", FALSE)
+              return()
+          }
         linearModel()
         return()
       }
@@ -185,10 +194,12 @@ linearModel <- function(){
     remove.cases <- if (remove == gettextRcmdr("<use all valid cases>") || remove == ""){
       "" 
     } else {
-      removeRows <- getCases(remove, remove=TRUE)
-      paste(", subset =", removeRows)
+#      removeRows <- getCases(remove, remove=TRUE)
+#      paste(", subset =", removeRows)
+        getCases(remove, remove=TRUE)
     }
-    if (remove.cases != "" && inherits(removeRows, "cases-error")){
+#    if (remove.cases != "" && inherits(removeRows, "cases-error")){
+    if (remove.cases != "" && inherits(remove.cases, "cases-error")){
       errorCondition(recall = linearModel,
                      message = removeRows,
                      model=TRUE)
@@ -201,9 +212,12 @@ linearModel <- function(){
       return()
     }
     formula <- paste(tclvalue(lhsVariable), tclvalue(rhsVariable), sep=" ~ ")
-    command <- paste("lm(", formula,
-                     ", data=", ActiveDataSet(), subset, weights, remove.cases, ")", sep="")
-    doItAndPrint(paste(modelValue, " <- ", command, sep = ""))
+    # command <- paste("lm(", formula,
+    #                  ", data=", ActiveDataSet(), subset, weights, remove.cases, ")", sep="")
+    # doItAndPrint(paste(modelValue, " <- ", command, sep = ""))
+    command <- Command("lm", formula, data=ActiveDataSet(), subset=subset, weights=weights,
+                       subset=remove.cases, to=modelValue)
+    doItAndPrint(command)
     doItAndPrint(paste("summary(", modelValue, ")", sep=""))
     activeModel(modelValue)
     if (subset != "" || remove.cases != "") putRcmdr("modelWithSubset", TRUE) else putRcmdr("modelWithSubset", FALSE)
@@ -336,6 +350,10 @@ generalizedLinearModel <- function(){
       if ("no" == tclvalue(checkReplace(modelValue, type=gettextRcmdr("Model")))){
         UpdateModelNumber(-1)
         closeDialog()
+        if (getRcmdr("onApplyCalled")){
+            putRcmdr("onApplyCalled", FALSE)
+            return()
+        }
         generalizedLinearModel()
         return()
       }
@@ -501,6 +519,10 @@ ordinalRegressionModel <- function(){
     if (is.element(modelValue, listProportionalOddsModels())) {
       if ("no" == tclvalue(checkReplace(modelValue, type=gettextRcmdr("Model")))){
         UpdateModelNumber(-1)
+          if (getRcmdr("onApplyCalled")){
+              putRcmdr("onApplyCalled", FALSE)
+              return()
+          }
         proportionalOddsModel()
         return()
       }
@@ -621,6 +643,10 @@ multinomialLogitModel <- function(){
     if (is.element(modelValue, listMultinomialLogitModels())) {
       if ("no" == tclvalue(checkReplace(modelValue, type=gettextRcmdr("Model")))){
         UpdateModelNumber(-1)
+          if (getRcmdr("onApplyCalled")){
+              putRcmdr("onApplyCalled", FALSE)
+              return()
+          }
         multinomialLogitModel()
         return()
       }
@@ -771,6 +797,10 @@ linearMixedModel <- function(){
     if (is.element(modelValue, listLMMs())) {
       if ("no" == tclvalue(checkReplace(modelValue, type=gettextRcmdr("Model")))){
         UpdateModelNumber(-1)
+          if (getRcmdr("onApplyCalled")){
+              putRcmdr("onApplyCalled", FALSE)
+              return()
+          }
         linearMixedModel()
         return()
       }
@@ -933,6 +963,10 @@ generalizedLinearMixedModel <- function(){
     if (is.element(modelValue, listGLMMs())) {
       if ("no" == tclvalue(checkReplace(modelValue, type=gettextRcmdr("Model")))){
         UpdateModelNumber(-1)
+          if (getRcmdr("onApplyCalled")){
+              putRcmdr("onApplyCalled", FALSE)
+              return()
+          }
         closeDialog()
         generalizedLinearMixedModel()
         return()
